// Copyright 1997 Ubilab, Union Bank of Switzerland. All rights reserved.
// This is part of the Geo system developed by Ubilab's DistrOOSE group.
// Check out: http://www.ubs.com/ubilab/Projects/se/se.html. GLAB-REH-97.

package Geo.Framework.AnyImpl;

import Geo.Framework.Any.*;



public abstract class AnyImpl implements Any, Cloneable {

	// class methods
	
	public static Any readFromString(String in, AnyContext context) {
		return AnyReaderDefImpl.read(in,context);
	}


	// comparing
	
	public final boolean equals(Object anObject) {
		try { return isEqual((Any) anObject); }
		catch (ClassCastException e) {
			return false;
		}
	}
	
	public boolean isEqual(Any anAny) {
		return isIdentical(anAny);
	}
	
	public final boolean notEqual(Any anAny) {
		return !isEqual(anAny);
	}	
	
	public final boolean isIdentical(Any anAny) {
		return this == anAny;
	}

	public final boolean notIdentical(Any anAny) {
		return !isIdentical(anAny);
	}
	
		
	// copying
	
	public final Object clone() {
		return (Object) deepCopy();
	}
	
	public Any shallowCopy() {
		try {
			return (Any) super.clone();
		}
		catch (CloneNotSupportedException e) {
			// this shouldn't happen, since we are Cloneable
			throw new InternalError();
		} 
	}
	
	public Any deepCopy() {
		try {
			return (Any) super.clone();
		}
		catch (CloneNotSupportedException e) {
			// this shouldn't happen, since we are Cloneable
			throw new InternalError();
		} 
	}
	
	public Any deepCopy(AnyContext anAnyContext) {
		String anyString = (new FastAnyWriterImpl()).writeAny(this);
		return (new FastAnyReaderImpl()).readAny(anyString, anAnyContext);	
	}
	
	
	// testing

	public boolean isObjectType() {
		return !isValueType();
	}

	public boolean isValueType() {
		return false;
	}
	
	public boolean isSimple() {
		return true;
	}
	
	
	// casting

	public final AnyArray asAnyArray() {
		try { return (AnyArray) this; }
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "AnyArray");
		}
	}
	
	public final AnyBoolean asAnyBoolean() {
		try { return (AnyBoolean) this; }
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "AnyBoolean");
		}
	}
	
	public final AnyDictionary asAnyDictionary() {
		try { return (AnyDictionary) this; }
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "AnyDictionary");
		}
	}
	
	public final AnyDouble asAnyDouble() {
		try { return (AnyDouble) this; }
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "AnyDouble");
		}
	}
	
	public final AnyFrame asAnyFrame() {
		try { return (AnyFrame) this; }
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "AnyFrame");
		}
	}
	
	public final AnyFrameDescriptor asAnyFrameDescriptor() {
		try { return (AnyFrameDescriptor) this; }
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "AnyFrameDescriptor");
		}
	}
	
	public final AnyInteger asAnyInteger() {
		try { return (AnyInteger) this; }
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "AnyInteger");
		}
	}
	
	public final AnySlotDescriptor asAnySlotDescriptor() {
		try { return (AnySlotDescriptor) this; } 
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "AnySlotDescriptor");
		}
	}
	
	public final AnyString asAnyString() {
		try { return (AnyString) this; }
		catch (ClassCastException e) { 
			throw new AnyClassCastException(this.getClass().getName(), "AnyString");
		}
	}	
	
	
	// accessing value
	
	public boolean asBoolean() {
		try { return ((AnyBoolean) this).asBoolean(); }
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "boolean"); 
		}
	}
	
	public double asDouble() {
		try { return ((AnyDouble) this).asDouble(); }
		catch (ClassCastException e) {
			throw new AnyClassCastException(this.getClass().getName(), "double");
		}
	}
	
	public int asInteger() {
		try {
			if (this instanceof AnyInteger) 
				return ((AnyInteger) this).asInteger();
			else 
				return ((AnyBoolean) this).asInteger(); }
		catch (ClassCastException e) {			
				throw new AnyClassCastException(this.getClass().getName(), "int");
		}
	}
	
	public String asString() {
		try { return ((AnyString) this).asString(); } 
		catch (ClassCastException e) { 
			throw new AnyClassCastException(this.getClass().getName(), "String");
		}
	}
	
		
	// printing 

	public void print() {
		String prettyString = (new PrettyAnyWriterImpl()).writeAny(this);
		System.out.println(prettyString);
	}
	
	public String printOnString(boolean fast) {
		AnyWriter writer;
		if (fast) 
			writer = new FastAnyWriterImpl();
		else 
			writer = new RobustAnyWriterImpl();
		return writer.writeAny(this);
	}	
	
	// misc
	
	public void addAllTo(AnyArray anArray) {
		anArray.append(this);				
	}
	
}

