// Copyright 1997 Ubilab, Union Bank of Switzerland. All rights reserved.
// This is part of the Geo system developed by Ubilab's DistrOOSE group.
// Check out: http://www.ubs.com/ubilab/Projects/se/se.html. GLAB-REH-97.

package Geo.Framework.AnyImpl;

import Geo.Framework.Any.Any;
import Geo.Framework.Any.AnyString;
import Geo.Framework.Any.AnySlotDescriptor;



public class AnySlotDescriptorImpl extends AnyImpl implements AnySlotDescriptor {

	// instance variables
	
	protected AnyString typeName;
	protected boolean singleValueFlag;
	protected boolean mayNotRemoveFlag;
	protected boolean mustHaveFlag;
	protected boolean notEditableFlag;


	// constructors
	
	public AnySlotDescriptorImpl(AnyString typeName) {
		this(typeName, false, false, false, false); 
	}

	public AnySlotDescriptorImpl(String typeName) {
		this(AnySap.stringValue(typeName));
	}
		
	public AnySlotDescriptorImpl(AnyString typeName, boolean isSingleValue, boolean isMayNotRemove,
								boolean isMustHave, boolean isNotEditable) {
		setTypeName(typeName);
		singleValueFlag = isSingleValue;
		mayNotRemoveFlag = isMayNotRemove;
		mustHaveFlag = isMustHave;
		notEditableFlag = isNotEditable;
	}
	
	public AnySlotDescriptorImpl(String typeName, boolean isSingleValue, boolean isMayNotRemove,
								boolean isMustHave, boolean isNotEditable) {
		this(AnySap.stringValue(typeName), isSingleValue, isMayNotRemove, isMustHave, isNotEditable);
	}
	
	
	// accessing

	public AnyString getTypeName() {
		return typeName;
	}	

	public boolean isMayNotRemove() {
		return mayNotRemoveFlag;
	}
	
	public boolean isMustHave() {
		return mustHaveFlag;
	}
	
	public boolean isNotEditable() {
		return notEditableFlag;
	}
	
	public boolean isSingleValue() {
		return singleValueFlag;
	}
	
	public String flagsToString() {
		StringBuffer buffer = new StringBuffer();
		if (isSingleValue())
			buffer.append(" SingleValue");
		if (isMayNotRemove())
			buffer.append(" MayNotRemove ");
		if (isMustHave())	
			buffer.append(" MustHave ");
		if (isNotEditable())
			buffer.append(" NotEditable ");
		return buffer.toString();
	}
	
	
	// setting
	
	public void setMayNotRemove() {
		mayNotRemoveFlag = true;
	}
	
	public void setMustHave() {
		mustHaveFlag = true;
	}
	
	public void setNotEditable() {
		notEditableFlag = true;
	}
	
	public void setSingleValue() {
		singleValueFlag = true;
	}
	
	public void setTypeName(AnyString anAnyString) {
		typeName = anAnyString;
	}
	
	public void resetMayNotRemove() {
		mayNotRemoveFlag = false;
	}
	
	public void resetMustHave() {
		mustHaveFlag = false;
	}
	
	public void resetNotEditable() {
		notEditableFlag = false;
	}
	
	public void resetSingleValue() {
		singleValueFlag = false;
	}
	
	
	// comparing
	
	public boolean isEqual(Any anAny) {
		AnySlotDescriptor anAnySlotDescriptor;
		if ( anAny == null || !(anAny instanceof AnySlotDescriptor) )
			return false;
		anAnySlotDescriptor = anAny.asAnySlotDescriptor();
		return getTypeName().isEqual(anAnySlotDescriptor.getTypeName()) 
			&& isMayNotRemove() == anAnySlotDescriptor.isMayNotRemove()
			&& isMustHave() == anAnySlotDescriptor.isMustHave()
			&& isNotEditable() == anAnySlotDescriptor.isNotEditable()
			&& isSingleValue() == anAnySlotDescriptor.isSingleValue();
	}
	
	public int hashCode() {
		return getTypeName().hashCode() + flagsToString().hashCode();
	}

}
